@@ -23,7 +23,9 @@
 #include "plm_version.h"
 #include "print_and_exit.h"
 #include "rtss.h"
+#include "rtss_contour.h"
 #include "rtss_roi.h"
+#include "string_util.h"
 
 PLMBASE_C_API bool 
 dcmtk_rtss_probe (const char *rtss_fn)
@@ -100,7 +102,7 @@ Dcmtk_loader::rtss_load (void)
             orc = seq->getItem(i)->findAndGetString (DCM_ROIName, val);
             lprintf ("Adding structure (%d), %s\n", structure_id, val);
             d_ptr->cxt->add_structure (
-                Pstring (val), Pstring (), structure_id);
+                std::string (val), std::string(), structure_id);
         }
     }
 
@@ -246,20 +248,15 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
 {
     OFCondition ofc;
     Rtss::Pointer& cxt = d_ptr->cxt;
-    const Metadata *rtss_metadata = 0;
-    if (d_ptr->dicom_metadata) {
-        rtss_metadata = d_ptr->dicom_metadata->get_rtss_metadata ();
+    Metadata::Pointer rtss_metadata;
+    if (d_ptr->rt_study_metadata) {
+        rtss_metadata = d_ptr->rt_study_metadata->get_rtss_metadata ();
     }
 
     /* Prepare structure set with slice uids */
-    const Slice_list *slice_list = d_ptr->dicom_metadata->get_slice_list ();
+    const Slice_list *slice_list = d_ptr->rt_study_metadata->get_slice_list ();
     cxt->apply_slice_list (slice_list);
 
-    /* Prepare output file */
-    Pstring rtss_fn;
-    rtss_fn.format ("%s/rtss.dcm", dicom_dir);
-    make_directory_recursive (rtss_fn);
-
     /* Prepare dcmtk */
     DcmFileFormat fileformat;
     DcmDataset *dataset = fileformat.getDataset();
@@ -268,18 +265,18 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
     /*     Part 1  -- General header                                     */
     /* ----------------------------------------------------------------- */
     dataset->putAndInsertOFStringArray(DCM_InstanceCreationDate, 
-        d_ptr->dicom_metadata->get_study_date());
+        d_ptr->rt_study_metadata->get_study_date());
     dataset->putAndInsertOFStringArray(DCM_InstanceCreationTime, 
-        d_ptr->dicom_metadata->get_study_time());
+        d_ptr->rt_study_metadata->get_study_time());
     dataset->putAndInsertOFStringArray(DCM_InstanceCreatorUID, 
         PLM_UID_PREFIX);
     dataset->putAndInsertString (DCM_SOPClassUID, UID_RTStructureSetStorage);
     dataset->putAndInsertString (DCM_SOPInstanceUID, 
-        d_ptr->dicom_metadata->get_rtss_instance_uid());
+        d_ptr->rt_study_metadata->get_rtss_instance_uid());
     dataset->putAndInsertOFStringArray (DCM_StudyDate, 
-        d_ptr->dicom_metadata->get_study_date());
+        d_ptr->rt_study_metadata->get_study_date());
     dataset->putAndInsertOFStringArray (DCM_StudyTime, 
-        d_ptr->dicom_metadata->get_study_time());
+        d_ptr->rt_study_metadata->get_study_time());
     dataset->putAndInsertOFStringArray (DCM_AccessionNumber, "");
     dataset->putAndInsertOFStringArray (DCM_Modality, "RTSTRUCT");
     dataset->putAndInsertString (DCM_Manufacturer, "Plastimatch");
@@ -304,18 +301,18 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
 #endif
 
     dataset->putAndInsertString (DCM_StudyInstanceUID, 
-        d_ptr->dicom_metadata->get_study_uid());
+        d_ptr->rt_study_metadata->get_study_uid());
     dataset->putAndInsertString (DCM_SeriesInstanceUID, 
-        d_ptr->dicom_metadata->get_rtss_series_uid());
+        d_ptr->rt_study_metadata->get_rtss_series_uid());
     dcmtk_copy_from_metadata (dataset, rtss_metadata, DCM_StudyID, "10001");
     dataset->putAndInsertString (DCM_SeriesNumber, "103");
     dataset->putAndInsertString (DCM_InstanceNumber, "1");
     dataset->putAndInsertString (DCM_StructureSetLabel, "AutoSS");
     dataset->putAndInsertString (DCM_StructureSetName, "AutoSS");
     dataset->putAndInsertOFStringArray (DCM_StructureSetDate, 
-        d_ptr->dicom_metadata->get_study_date());
+        d_ptr->rt_study_metadata->get_study_date());
     dataset->putAndInsertOFStringArray (DCM_StructureSetTime, 
-        d_ptr->dicom_metadata->get_study_time());
+        d_ptr->rt_study_metadata->get_study_time());
 
     /* ----------------------------------------------------------------- */
     /*     Part 2  -- UID's for CT series                                */
@@ -325,7 +322,7 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
     dataset->findOrCreateSequenceItem (
         DCM_ReferencedFrameOfReferenceSequence, rfor_item, -2);
     rfor_item->putAndInsertString (DCM_FrameOfReferenceUID, 
-        d_ptr->dicom_metadata->get_frame_of_reference_uid());
+        d_ptr->rt_study_metadata->get_frame_of_reference_uid());
     dataset->findAndGetSequence (
         DCM_ReferencedFrameOfReferenceSequence, rfor_seq);
     DcmItem *rtrstudy_item = 0;
@@ -335,14 +332,14 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
         DCM_ReferencedSOPClassUID, 
         UID_RETIRED_StudyComponentManagementSOPClass);
     rtrstudy_item->putAndInsertString (
-        DCM_ReferencedSOPInstanceUID, d_ptr->dicom_metadata->get_study_uid());
+        DCM_ReferencedSOPInstanceUID, d_ptr->rt_study_metadata->get_study_uid());
     DcmItem *rtrseries_item = 0;
     rtrstudy_item->findOrCreateSequenceItem (
         DCM_RTReferencedSeriesSequence, rtrseries_item, -2);
     rtrseries_item->putAndInsertString (
-        DCM_SeriesInstanceUID, d_ptr->dicom_metadata->get_ct_series_uid());
+        DCM_SeriesInstanceUID, d_ptr->rt_study_metadata->get_ct_series_uid());
 
-    for (int k = 0; k < d_ptr->dicom_metadata->num_slices(); k++) {
+    for (int k = 0; k < d_ptr->rt_study_metadata->num_slices(); k++) {
         DcmItem *ci_item = 0;
         rtrseries_item->findOrCreateSequenceItem (
             DCM_ContourImageSequence, ci_item, -2);
@@ -350,7 +347,7 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
             DCM_ReferencedSOPClassUID, UID_CTImageStorage);
         ci_item->putAndInsertString (
             DCM_ReferencedSOPInstanceUID, 
-            d_ptr->dicom_metadata->get_slice_uid (k));
+            d_ptr->rt_study_metadata->get_slice_uid (k));
     }
 
     /* ----------------------------------------------------------------- */
@@ -358,14 +355,15 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
     /* ----------------------------------------------------------------- */
     for (size_t i = 0; i < cxt->num_structures; i++) {
         DcmItem *ssroi_item = 0;
-        Pstring tmp;
+        std::string tmp;
         dataset->findOrCreateSequenceItem (
             DCM_StructureSetROISequence, ssroi_item, -2);
-        tmp.format ("%d", cxt->slist[i]->id);
+        tmp = string_format ("%d", cxt->slist[i]->id);
         ssroi_item->putAndInsertString (DCM_ROINumber, tmp.c_str());
         ssroi_item->putAndInsertString (DCM_ReferencedFrameOfReferenceUID,
-            d_ptr->dicom_metadata->get_frame_of_reference_uid());
-        ssroi_item->putAndInsertString (DCM_ROIName, cxt->slist[i]->name);
+            d_ptr->rt_study_metadata->get_frame_of_reference_uid());
+        ssroi_item->putAndInsertString (DCM_ROIName, 
+            cxt->slist[i]->name.c_str());
         ssroi_item->putAndInsertString (DCM_ROIGenerationAlgorithm, "");
     }
 
@@ -375,10 +373,9 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
     for (size_t i = 0; i < cxt->num_structures; i++) {
 	Rtss_roi *curr_structure = cxt->slist[i];
         DcmItem *roic_item = 0;
-	Pstring tmp;
         dataset->findOrCreateSequenceItem (
             DCM_ROIContourSequence, roic_item, -2);
-        curr_structure->get_dcm_color_string (&tmp);
+        std::string tmp = curr_structure->get_dcm_color_string ();
         roic_item->putAndInsertString (DCM_ROIDisplayColor, tmp.c_str());
 	for (size_t j = 0; j < curr_structure->num_contours; j++) {
 	    Rtss_contour *curr_contour = curr_structure->pslist[j];
@@ -403,7 +400,7 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
                 DCM_ContourSequence, c_item, -2);
 
 	    /* ContourImageSequence */
-	    if (curr_contour->ct_slice_uid.not_empty()) {
+	    if (curr_contour->ct_slice_uid != "") {
                 DcmItem *ci_item = 0;
                 c_item->findOrCreateSequenceItem (
                     DCM_ContourImageSequence, ci_item, -2);
@@ -418,27 +415,26 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
                 "CLOSED_PLANAR");
 
             /* NumberOfContourPoints */
-            tmp.format ("%d", curr_contour->num_vertices);
-            c_item->putAndInsertString (DCM_NumberOfContourPoints, tmp);
+            tmp = string_format ("%d", curr_contour->num_vertices);
+            c_item->putAndInsertString (DCM_NumberOfContourPoints, tmp.c_str());
 
 	    /* ContourData */
-            tmp.format ("%g\\%g\\%g", 
+            tmp = string_format ("%.8g\\%.8g\\%.8g", 
                 curr_contour->x[0],
                 curr_contour->y[0],
                 curr_contour->z[0]);
 	    for (int k = 1; k < curr_contour->num_vertices; k++) {
-                Pstring tmp2;
-                tmp2.format ("\\%g\\%g\\%g",
+                std::string tmp2 = string_format ("\\%.8g\\%.8g\\%.8g",
 		    curr_contour->x[k],
 		    curr_contour->y[k],
 		    curr_contour->z[k]);
                 tmp += tmp2;
 	    }
-            c_item->putAndInsertString (DCM_ContourData, tmp);
+            c_item->putAndInsertString (DCM_ContourData, tmp.c_str());
         }
 
-        tmp.format ("%d", (int) curr_structure->id);
-        roic_item->putAndInsertString (DCM_ReferencedROINumber, tmp);
+        tmp = string_format ("%d", (int) curr_structure->id);
+        roic_item->putAndInsertString (DCM_ReferencedROINumber, tmp.c_str());
     }
 
     /* ----------------------------------------------------------------- */
@@ -446,7 +442,7 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
     /* ----------------------------------------------------------------- */
     for (size_t i = 0; i < cxt->num_structures; i++) {
 	Rtss_roi *curr_structure = cxt->slist[i];
-	Pstring tmp;
+        std::string tmp;
 
         /* RTROIObservationsSequence */
         DcmItem *rtroio_item = 0;
@@ -454,20 +450,19 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
             DCM_RTROIObservationsSequence, rtroio_item, -2);
 
 	/* ObservationNumber */
-        tmp.format ("%d", (int) curr_structure->id);
-	rtroio_item->putAndInsertString (DCM_ObservationNumber, tmp);
+        tmp = string_format ("%d", (int) curr_structure->id);
+	rtroio_item->putAndInsertString (DCM_ObservationNumber, tmp.c_str());
 	/* ReferencedROINumber */
-	rtroio_item->putAndInsertString (DCM_ReferencedROINumber, tmp);
+	rtroio_item->putAndInsertString (DCM_ReferencedROINumber, tmp.c_str());
 	/* ROIObservationLabel */
         if (curr_structure->name.length() <= 16) {
             rtroio_item->putAndInsertString (DCM_ROIObservationLabel, 
-                (const char*) curr_structure->name);
+                curr_structure->name.c_str());
         } else {
             /* VR is SH, max length 16 */
-            Pstring tmp_name = curr_structure->name;
-            tmp_name.trunc (16);
+            std::string tmp_name = curr_structure->name.substr (0, 16);
             rtroio_item->putAndInsertString (DCM_ROIObservationLabel, 
-                (const char*) tmp_name);
+                tmp_name.c_str());
         }
 	/* RTROIInterpretedType */
 	rtroio_item->putAndInsertString (DCM_RTROIInterpretedType, "");
@@ -478,6 +473,15 @@ Dcmtk_rt_study::save_rtss (const char *dicom_dir)
     /* ----------------------------------------------------------------- */
     /*     Write the output file                                         */
     /* ----------------------------------------------------------------- */
+    std::string rtss_fn;
+    if (d_ptr->filenames_with_uid) {
+        rtss_fn = string_format ("%s/rtss_%s.dcm", dicom_dir, 
+            d_ptr->rt_study_metadata->get_rtss_series_uid());
+    } else {
+        rtss_fn = string_format ("%s/rtss.dcm", dicom_dir);
+    }
+    make_parent_directories (rtss_fn);
+
     ofc = fileformat.saveFile (rtss_fn.c_str(), EXS_LittleEndianExplicit);
     if (ofc.bad()) {
         print_and_exit ("Error: cannot write DICOM RTSTRUCT (%s)\n", 
