@@ -5,6 +5,7 @@
 #define _parameter_parser_h_
 
 #include "plmbase_config.h"
+#include "plm_return_code.h"
 
 class PLMBASE_API Parameter_parser {
 public:
@@ -13,9 +14,11 @@ public:
     bool key_regularization;
 public:
     /* Callbacks */
-    virtual int process_section (
+    virtual Plm_return_code begin_section (
         const std::string& section) = 0;
-    virtual int process_key_value (
+    virtual Plm_return_code end_section (
+        const std::string& section) = 0;
+    virtual Plm_return_code set_key_value (
         const std::string& section, 
         const std::string& key, 
         const std::string& val) = 0;
@@ -27,15 +30,18 @@ public:
     );
 
     /* Return zero if config string is correctly parsed */
-    int parse_config_string (
+    Plm_return_code parse_config_string (
         const char* config_string
     );
-    int parse_config_string (
+    Plm_return_code parse_config_string (
         const std::string& config_string
     );
-    int parse_config_file (
+    Plm_return_code parse_config_file (
         const char* config_fn
     );
+    Plm_return_code parse_config_file (
+        const std::string& config_fn
+    );
 };
 
 #endif
