@@ -5,6 +5,7 @@
 #define _plm_image_type_h_
 
 #include "plmbase_config.h"
+#include <string>
 
 /* We only deal with these kinds of images. */
 enum Plm_image_type {
@@ -29,6 +30,7 @@ enum Plm_image_type {
     PLM_IMG_TYPE_GPUIT_UCHAR_VEC
 };
 
+PLMBASE_API Plm_image_type plm_image_type_parse (const std::string& string);
 PLMBASE_API Plm_image_type plm_image_type_parse (const char* string);
 PLMBASE_API char* plm_image_type_string (Plm_image_type type);
 PLMBASE_API char* plm_image_type_string_simple (Plm_image_type type);
