@@ -52,23 +52,33 @@ Volume::Volume () {
 
 Volume::Volume (
     const plm_long dim[3], 
-    const float offset[3], 
+    const float origin[3], 
     const float spacing[3], 
     const float direction_cosines[9], 
     enum Volume_pixel_type vox_type, 
     int vox_planes
 ) {
-    init ();
-    create (dim, offset, spacing, direction_cosines, vox_type, 
+    create (dim, origin, spacing, direction_cosines, vox_type, 
         vox_planes);
 }
 
+Volume::Volume (
+    const plm_long dim[3], 
+    const float origin[3], 
+    const float spacing[3], 
+    const Direction_cosines& direction_cosines, 
+    enum Volume_pixel_type vox_type, 
+    int vox_planes
+) {
+    create (dim, origin, spacing, direction_cosines.get_matrix(), 
+        vox_type, vox_planes);
+}
+
 Volume::Volume (
     const Volume_header& vh, 
     enum Volume_pixel_type vox_type, 
     int vox_planes
 ) {
-    init ();
     create (vh, vox_type, vox_planes);
 }
 
@@ -86,9 +96,10 @@ Volume::~Volume ()
 void
 Volume::init ()
 {
+	
     for (int d = 0; d < 3; d++) {
         dim[d] = 0;
-        offset[d] = 0;
+        origin[d] = 0;
         spacing[d] = 0;
     }
     for (int i = 0; i < 3; i++) {
@@ -136,17 +147,18 @@ Volume::allocate (void)
 
 void 
 Volume::create (
-    const plm_long dim[3], 
-    const float offset[3], 
+    const plm_long new_dim[3], 
+    const float origin[3], 
     const float spacing[3], 
     const float direction_cosines[9], 
     enum Volume_pixel_type vox_type, 
     int vox_planes
 )
 {
+    init ();
     for (int i = 0; i < 3; i++) {
-	this->dim[i] = dim[i];
-	this->offset[i] = offset[i];
+	this->dim[i] = new_dim[i];
+	this->origin[i] = origin[i];
 	this->spacing[i] = spacing[i];
     }
     this->npix = this->dim[0] * this->dim[1] * this->dim[2];
@@ -205,23 +217,17 @@ Volume::create (
 const float*
 Volume::get_origin ()
 {
-    return this->offset;
+    return this->origin;
 }
 
 void
 Volume::set_origin (const float origin[3])
 {
     for (int d = 0; d < 3; d++) {
-        this->offset[d] = origin[d];
+        this->origin[d] = origin[d];
     }
 }
 
-float*
-Volume::get_direction_cosines ()
-{
-    return this->direction_cosines;
-}
-
 void 
 Volume::set_direction_cosines (
     const float direction_cosines[9]
@@ -267,7 +273,7 @@ Volume*
 volume_clone_empty (Volume* ref)
 {
     Volume* vout;
-    vout = new Volume (ref->dim, ref->offset, ref->spacing, 
+    vout = new Volume (ref->dim, ref->origin, ref->spacing, 
 	ref->direction_cosines, ref->pix_type, ref->vox_planes);
     return vout;
 }
@@ -276,7 +282,7 @@ Volume*
 volume_clone (const Volume* ref)
 {
     Volume* vout;
-    vout = new Volume (ref->dim, ref->offset, ref->spacing, 
+    vout = new Volume (ref->dim, ref->origin, ref->spacing, 
 	ref->direction_cosines, ref->pix_type, ref->vox_planes);
     switch (ref->pix_type) {
     case PT_UCHAR:
@@ -689,7 +695,7 @@ Volume::clone (Volume_pixel_type new_type) const
 {
     Volume::Pointer vol_out = Volume::New ();
     vol_out->create (
-        this->dim, this->offset, this->spacing, 
+        this->dim, this->origin, this->spacing, 
 	this->direction_cosines, new_type,
         this->vox_planes);
     switch (new_type) {
@@ -788,12 +794,13 @@ Volume::get_ijk_value (const float ijk[3])
     return val;
 }
 
+/* GCS FIX: These functions are incorrect, they ignore direction cosines */
 void 
 Volume::get_xyz_from_ijk (double xyz[3], const int ijk[3])
 {
-    xyz[0] = this->offset[0] + ijk[0] * this->spacing[0];
-    xyz[1] = this->offset[1] + ijk[1] * this->spacing[1];
-    xyz[2] = this->offset[2] + ijk[2] * this->spacing[2];
+    xyz[0] = this->origin[0] + ijk[0] * this->spacing[0];
+    xyz[1] = this->origin[1] + ijk[1] * this->spacing[1];
+    xyz[2] = this->origin[2] + ijk[2] * this->spacing[2];
 }
 
 void
@@ -803,13 +810,14 @@ Volume::get_ijk_from_xyz (float ijk[3], const float xyz[3], bool* in)
 
     for (int i = 0; i < 3; i++)
     {
-        ijk[i] = (float) floor(xyz[i]-this->offset[i])/this->spacing[i];
-        if (ijk[i] < 0 || ijk[i] >= this->dim[i])
+        ijk[i] = (float) (xyz[i]-this->origin[i])/this->spacing[i];
+        if (ijk[i] < 0 || ijk[i] >= this->dim[i] -1)
         {
             *in = false;
             return;
         }
     }
+    return;
 }
 
 void
@@ -819,7 +827,7 @@ Volume::get_ijk_from_xyz (int ijk[3], const float xyz[3], bool* in)
 
     for (int i = 0; i < 3; i++)
     {
-        ijk[i] = (int) floor(xyz[i]-this->offset[i])/this->spacing[i];
+        ijk[i] = (int) floor(xyz[i]-this->origin[i])/this->spacing[i];
         if (ijk[i] < 0 || ijk[i] >= this->dim[i])
         {
             *in = false;
@@ -828,6 +836,15 @@ Volume::get_ijk_from_xyz (int ijk[3], const float xyz[3], bool* in)
     }
 }
 
+bool
+Volume::is_inside (const float ijk[3]) const
+{
+    if (ijk[0] <= -0.5 || ijk[0] >= this->dim[0] - 0.5) return false;
+    if (ijk[1] <= -0.5 || ijk[1] >= this->dim[1] - 0.5) return false;
+    if (ijk[2] <= -0.5 || ijk[2] >= this->dim[2] - 0.5) return false;
+    return true;
+}
+
 void
 Volume::scale_inplace (float scale)
 {
@@ -852,9 +869,9 @@ Volume::debug ()
 	(int) dim[2]
     );
     lprintf ("org:%f %f %f\n",
-	offset[0],
-	offset[1],
-	offset[2]
+	origin[0],
+	origin[1],
+	origin[2]
     );
     lprintf ("spac:%f %f %f\n",
 	spacing[0],
@@ -878,9 +895,9 @@ void
 Volume::direction_cosines_debug ()
 {
     lprintf ("org:%f %f %f\n",
-	offset[0],
-	offset[1],
-	offset[2]
+	origin[0],
+	origin[1],
+	origin[2]
     );
     lprintf ("spac:%f %f %f\n",
 	spacing[0],
@@ -922,119 +939,6 @@ Volume::direction_cosines_debug ()
     );
 }
 
-/* In mm coordinates */
-void
-volume_calc_grad_dcos (Volume* vout, const Volume* vref)
-{
-    plm_long i, j, k;
-    plm_long i_n, j_n, k_n;       /* n is next */
-    int i_p, j_p, k_p;          /* p is prev */
-    plm_long gi, gj, gk;
-    plm_long idx_p, idx_n;
-    float *out_img, *ref_img;
-
-    out_img = (float*) vout->img;
-    ref_img = (float*) vref->img;
-
-    const float *inv_dc = vref->direction_cosines.get_inverse();
-
-    plm_long v = 0;
-    for (k = 0; k < vref->dim[2]; k++) {
-	k_p = k - 1;
-	k_n = k + 1;
-	if (k == 0) k_p = 0;
-	if (k == vref->dim[2]-1) k_n = vref->dim[2]-1;
-	for (j = 0; j < vref->dim[1]; j++) {
-	    j_p = j - 1;
-	    j_n = j + 1;
-	    if (j == 0) j_p = 0;
-	    if (j == vref->dim[1]-1) j_n = vref->dim[1]-1;
-	    for (i = 0; i < vref->dim[0]; i++, v++) {
-		float diff;
-		i_p = i - 1;
-		i_n = i + 1;
-		if (i == 0) i_p = 0;
-		if (i == vref->dim[0]-1) i_n = vref->dim[0]-1;
-		
-		gi = 3 * v + 0;
-		gj = 3 * v + 1;
-		gk = 3 * v + 2;
-		out_img[gi] = 0.f;
-		out_img[gj] = 0.f;
-		out_img[gk] = 0.f;
-
-#if defined (commentout)		
-		idx_p = volume_index (vref->dim, i_p, j, k);
-		idx_n = volume_index (vref->dim, i_n, j, k);
-		diff = (float) (ref_img[idx_n] - ref_img[idx_p]) 
-		    / 2.0 / vref->spacing[0];
-		out_img[gi] += diff * vref->direction_cosines[0*3+0];
-		out_img[gj] += diff * vref->direction_cosines[0*3+1];
-		out_img[gk] += diff * vref->direction_cosines[0*3+2];
-
-		idx_p = volume_index (vref->dim, i, j_p, k);
-		idx_n = volume_index (vref->dim, i, j_n, k);
-		diff = (float) (ref_img[idx_n] - ref_img[idx_p]) 
-		    / 2.0 / vref->spacing[1];
-		out_img[gi] += diff * vref->direction_cosines[1*3+0];
-		out_img[gj] += diff * vref->direction_cosines[1*3+1];
-		out_img[gk] += diff * vref->direction_cosines[1*3+2];
-
-		idx_p = volume_index (vref->dim, i, j, k_p);
-		idx_n = volume_index (vref->dim, i, j, k_n);
-		diff = (float) (ref_img[idx_n] - ref_img[idx_p]) 
-		    / 2.0 / vref->spacing[2];
-		out_img[gi] += diff * vref->direction_cosines[2*3+0];
-		out_img[gj] += diff * vref->direction_cosines[2*3+1];
-		out_img[gk] += diff * vref->direction_cosines[2*3+2];
-#endif
-
-		idx_p = volume_index (vref->dim, i_p, j, k);
-		idx_n = volume_index (vref->dim, i_n, j, k);
-		diff = (float) (ref_img[idx_n] - ref_img[idx_p]) 
-		    / 2.0 / vref->spacing[0];
-		out_img[gi] += diff * inv_dc[0*3+0];
-		out_img[gj] += diff * inv_dc[1*3+0];
-		out_img[gk] += diff * inv_dc[2*3+0];
-
-		idx_p = volume_index (vref->dim, i, j_p, k);
-		idx_n = volume_index (vref->dim, i, j_n, k);
-		diff = (float) (ref_img[idx_n] - ref_img[idx_p]) 
-		    / 2.0 / vref->spacing[1];
-		out_img[gi] += diff * inv_dc[0*3+1];
-		out_img[gj] += diff * inv_dc[1*3+1];
-		out_img[gk] += diff * inv_dc[2*3+1];
-
-		idx_p = volume_index (vref->dim, i, j, k_p);
-		idx_n = volume_index (vref->dim, i, j, k_n);
-		diff = (float) (ref_img[idx_n] - ref_img[idx_p]) 
-		    / 2.0 / vref->spacing[2];
-		out_img[gi] += diff * inv_dc[0*3+2];
-		out_img[gj] += diff * inv_dc[1*3+2];
-		out_img[gk] += diff * inv_dc[2*3+2];
-	    }
-	}
-    }
-    lprintf ("volume_calc_grad_dcos complete.\n");
-}
-
-void
-volume_calc_grad (Volume* vout, const Volume* vref)
-{
-    volume_calc_grad_dcos (vout, vref);
-}
-
-Volume* 
-volume_make_gradient (Volume* ref)
-{
-    Volume *grad;
-    grad = new Volume (ref->dim, ref->offset, ref->spacing, 
-	ref->direction_cosines, PT_VF_FLOAT_INTERLEAVED, 3);
-    volume_calc_grad (grad, ref);
-
-    return grad;
-}
-
 // Computes the intensity differences between two images
 Volume*
 volume_difference (Volume* vol, Volume* warped)
@@ -1054,7 +958,7 @@ volume_difference (Volume* vol, Volume* warped)
 
     for(i=0;i<3; i++){
 	temp->dim[i] = vol->dim[i];
-	temp->offset[i] = vol->offset[i];
+	temp->origin[i] = vol->origin[i];
 	temp->spacing[i] = vol->spacing[i];
     }
 
