@@ -24,7 +24,6 @@
 #include "plm_image_header.h"
 #include "plm_math.h"
 #include "print_and_exit.h"
-#include "pstring.h"
 #include "string_util.h"
 #include "volume.h"
 #include "volume_header.h"
@@ -35,7 +34,7 @@
 static void
 itk_bsp_set_grid_img (Xform *xf,
     const Plm_image_header* pih,
-    float* grid_spac);
+    const float* grid_spac);
 static void load_gpuit_bsp (Xform *xf, const char* fn);
 static void itk_xform_load (Xform *xf, const char* fn);
 
@@ -337,12 +336,6 @@ Xform::load (const char* fn)
     }
 }
 
-void
-Xform::load (const Pstring& fn)
-{
-    this->load ((const char*) fn);
-}
-
 void
 Xform::load (const std::string& fn)
 {
@@ -355,6 +348,12 @@ xform_load (Xform *xf, const char* fn)
     xf->load (fn);
 }
 
+void
+xform_load (Xform *xf, const std::string& fn)
+{
+    xf->load (fn);
+}
+
 Xform::Pointer xform_load (const char* fn)
 {
     Xform::Pointer xf = Xform::New ();
@@ -523,12 +522,6 @@ Xform::save (const char* fn)
     }
 }
 
-void
-Xform::save (const Pstring& fn)
-{
-    this->save ((const char*) fn);
-}
-
 void
 Xform::save (const std::string& fn)
 {
@@ -541,6 +534,12 @@ xform_save (Xform *xf, const char* fn)
     xf->save (fn);
 }
 
+void
+xform_save (Xform *xf, const std::string& fn)
+{
+    xf->save (fn.c_str());
+}
+
 /* -----------------------------------------------------------------------
    Defaults
    ----------------------------------------------------------------------- */
@@ -655,7 +654,7 @@ bsp_grid_from_img_grid (
     BsplineTransformType::RegionType& bsp_region,       /* Output */
     BsplineTransformType::DirectionType& bsp_direction, /* Output */
     const Plm_image_header* pih,                        /* Input */
-    float* grid_spac)                                   /* Input */
+    const float* grid_spac)                                   /* Input */
 {
     BsplineTransformType::RegionType::SizeType bsp_size;
 
@@ -682,7 +681,7 @@ static void
 itk_bsp_set_grid_img (
     Xform *xf,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     BsplineTransformType::OriginType bsp_origin;
     BsplineTransformType::SpacingType bsp_spacing;
@@ -703,7 +702,7 @@ xform_trn_to_itk_bsp_bulk (
     Xform *xf_out, 
     const Xform* xf_in,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     xform_itk_bsp_init_default (xf_out);
     itk_bsp_set_grid_img (xf_out, pih, grid_spac);
@@ -715,7 +714,7 @@ xform_vrs_to_itk_bsp_bulk (
     Xform *xf_out, 
     const Xform* xf_in,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     xform_itk_bsp_init_default (xf_out);
     itk_bsp_set_grid_img (xf_out, pih, grid_spac);
@@ -727,7 +726,7 @@ xform_quat_to_itk_bsp_bulk (
     Xform *xf_out, 
     const Xform* xf_in,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     xform_itk_bsp_init_default (xf_out);
     itk_bsp_set_grid_img (xf_out, pih, grid_spac);
@@ -739,7 +738,7 @@ xform_aff_to_itk_bsp_bulk (
     Xform *xf_out, 
     const Xform* xf_in,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     xform_itk_bsp_init_default (xf_out);
     itk_bsp_set_grid_img (xf_out, pih, grid_spac);
@@ -751,7 +750,7 @@ static void
 xform_any_to_itk_bsp_nobulk (
     Xform *xf_out, Xform* xf_in,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     int d;
     Xform xf_tmp;
@@ -911,7 +910,7 @@ xform_itk_bsp_to_itk_bsp (
     Xform *xf_out, 
     const Xform* xf_in,
     const Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     BsplineTransformType::Pointer bsp_old = xf_in->get_itk_bsp();
 
@@ -1076,7 +1075,7 @@ xform_gpuit_bsp_to_itk_bsp (
     const Xform* xf_in,
     const Plm_image_header* pih,
     const ImageRegionType* roi, /* Not yet used */
-    float* grid_spac)
+    const float* grid_spac)
 {
     Xform xf_tmp;
 
@@ -1156,10 +1155,11 @@ xform_itk_bsp_to_itk_vf (Xform* xf_in, const Plm_image_header* pih)
 }
 
 static DeformationFieldType::Pointer 
-xform_itk_vf_to_itk_vf (DeformationFieldType::Pointer vf, Plm_image_header* pih)
+xform_itk_vf_to_itk_vf (
+    const DeformationFieldType::Pointer& vf, 
+    Plm_image_header* pih)
 {
-    vf = vector_resample_image (vf, pih);
-    return vf;
+    return vector_resample_image (vf, (const Plm_image_header*) pih);
 }
 
 /* Here what we're going to do is use GPUIT library to interpolate the 
@@ -1247,7 +1247,7 @@ xform_gpuit_vf_to_itk_vf (
    Conversion to gpuit_bsp
    ----------------------------------------------------------------------- */
 static Bspline_xform*
-create_gpuit_bxf (Plm_image_header* pih, float* grid_spac)
+create_gpuit_bxf (Plm_image_header* pih, const float* grid_spac)
 {
     int d;
     Bspline_xform* bxf = new Bspline_xform;
@@ -1284,7 +1284,7 @@ void
 xform_any_to_gpuit_bsp (
     Xform* xf_out, Xform* xf_in, 
     Plm_image_header* pih, 
-    float* grid_spac)
+    const float* grid_spac)
 {
     Xform xf_tmp;
     ImageRegionType roi;
@@ -1318,7 +1318,7 @@ xform_gpuit_bsp_to_gpuit_bsp (
     Xform* xf_out, 
     Xform* xf_in, 
     Plm_image_header* pih, 
-    float* grid_spac
+    const float* grid_spac
 )
 {
     Xform xf_tmp;
@@ -1351,7 +1351,7 @@ xform_gpuit_vf_to_gpuit_bsp (
     Xform* xf_out, 
     Xform* xf_in, 
     Plm_image_header* pih, 
-    float* grid_spac
+    const float* grid_spac
 )
 {
     /* Convert gpuit_vf to itk_vf, then convert itk_vf to gpuit_bsp */
@@ -1368,9 +1368,8 @@ xform_itk_any_to_gpuit_vf (
     itk::Transform<double,3,3>* xf,
     const Plm_image_header* pih)
 {
-    Volume_header vh = pih->get_volume_header();
-    Volume::Pointer vf_out = Volume::New (new Volume (
-            vh, PT_VF_FLOAT_INTERLEAVED, 3));
+    Volume_header vh (pih);
+    Volume::Pointer vf_out = Volume::New (vh, PT_VF_FLOAT_INTERLEAVED, 3);
     float* img = (float*) vf_out->img;
 
     DoublePoint3DType fixed_point;
@@ -1401,7 +1400,7 @@ xform_gpuit_vf_to_gpuit_vf (
     const Plm_image_header *pih)
 {
     Volume::Pointer vf_out;
-    Volume_header vh = pih->get_volume_header();
+    Volume_header vh (pih);
     vf_out = volume_resample (vf_in, &vh);
     return vf_out;
 }
@@ -1412,8 +1411,8 @@ xform_gpuit_bsp_to_gpuit_vf (const Xform *xf_in, const Plm_image_header *pih)
     Bspline_xform* bxf = xf_in->get_gpuit_bsp();
     Volume::Pointer vf_out;
 
-    Volume_header vh = pih->get_volume_header ();
-    vf_out = Volume::New (new Volume (vh, PT_VF_FLOAT_INTERLEAVED, 3));
+    Volume_header vh (pih);
+    vf_out = Volume::New (vh, PT_VF_FLOAT_INTERLEAVED, 3);
     bspline_interpolate_vf (vf_out.get(), bxf);
     return vf_out;
 }
@@ -1422,9 +1421,8 @@ Volume::Pointer
 xform_itk_vf_to_gpuit_vf (
     DeformationFieldType::Pointer itk_vf, const Plm_image_header *pih)
 {
-    Volume_header vh = pih->get_volume_header();
-    Volume::Pointer vf_out = Volume::New (new Volume (
-            vh, PT_VF_FLOAT_INTERLEAVED, 3));
+    Volume_header vh (pih);
+    Volume::Pointer vf_out = Volume::New (vh, PT_VF_FLOAT_INTERLEAVED, 3);
     float* img = (float*) vf_out->img;
     FloatVector3DType displacement;
 
@@ -1583,7 +1581,7 @@ xform_to_itk_bsp (
     Xform *xf_out, 
     const Xform *xf_in, 
     Plm_image_header* pih,
-    float* grid_spac
+    const float* grid_spac
 )
 {
     BsplineTransformType::Pointer bsp;
@@ -1631,7 +1629,7 @@ Xform::Pointer
 xform_to_itk_bsp (
     const Xform::Pointer& xf_in, 
     Plm_image_header* pih,
-    float* grid_spac
+    const float* grid_spac
 )
 {
     Xform::Pointer xf_out = Xform::New ();
@@ -1643,7 +1641,7 @@ void
 xform_to_itk_bsp_nobulk (
     Xform *xf_out, Xform *xf_in, 
     Plm_image_header* pih,
-    float* grid_spac)
+    const float* grid_spac)
 {
     switch (xf_in->m_type) {
     case XFORM_NONE:
@@ -1687,7 +1685,7 @@ Xform::Pointer
 xform_to_itk_bsp_nobulk (
     const Xform::Pointer& xf_in, 
     Plm_image_header* pih,
-    float* grid_spac
+    const float* grid_spac
 )
 {
     Xform::Pointer xf_out = Xform::New ();
@@ -1757,7 +1755,7 @@ xform_to_itk_vf (const Xform::Pointer& xf_in, Plm_image_header* pih)
 
 void
 xform_to_gpuit_bsp (Xform* xf_out, Xform* xf_in, Plm_image_header* pih, 
-    float* grid_spac)
+    const float* grid_spac)
 {
     switch (xf_in->m_type) {
     case XFORM_NONE:
