@@ -12,20 +12,25 @@
 #include "plm_image.h"
 #include "plm_math.h"
 #include "print_and_exit.h"
-#include "pstring.h"
 #include "rt_study.h"
+#include "string_util.h"
 #include "synthetic_mha.h"
 
-typedef struct synthetic_mha_main_parms Synthetic_mha_main_parms;
-struct synthetic_mha_main_parms {
-    Pstring output_fn;
-    Pstring output_dose_img_fn;
-    Pstring output_ss_img_fn;
-    Pstring output_ss_list_fn;
-    Pstring output_dicom;
+class Synthetic_mha_main_parms
+{
+public:
+    std::string output_fn;
+    std::string output_dose_img_fn;
+    std::string output_ss_img_fn;
+    std::string output_ss_list_fn;
+    std::string output_dicom;
     Synthetic_mha_parms sm_parms;
-
+    bool dicom_with_uids;
     std::vector<std::string> m_metadata;
+public:
+    Synthetic_mha_main_parms () {
+        dicom_with_uids = true;
+    }
 };
 
 void
@@ -35,55 +40,30 @@ do_synthetic_mha (Synthetic_mha_main_parms *parms)
 
     /* Create image */
     Rt_study rtds;
-    if (parms->output_dicom.not_empty() 
-        || parms->output_ss_img_fn.not_empty()
-        || parms->output_ss_list_fn.not_empty())
+    if (parms->output_dicom != "" 
+        || parms->output_ss_img_fn != ""
+        || parms->output_ss_list_fn != "")
     {
         sm_parms->m_want_ss_img = true;
     }
-    if (parms->output_dicom.not_empty() 
-        || parms->output_dose_img_fn.not_empty())
+    if (parms->output_dicom != "" 
+        || parms->output_dose_img_fn != "")
     {
         sm_parms->m_want_dose_img = true;
     }
     synthetic_mha (&rtds, sm_parms);
 
     /* metadata */
-    rtds.set_user_metadata (parms->m_metadata);
+    rtds.set_study_metadata (parms->m_metadata);
 
     /* Save to file */
     FloatImageType::Pointer img = rtds.get_image()->itk_float();
-    if (parms->output_fn.not_empty()) {
-        switch (sm_parms->output_type) {
-        case PLM_IMG_TYPE_ITK_UCHAR:
-            itk_image_save_uchar (img, (const char*) parms->output_fn);
-            break;
-        case PLM_IMG_TYPE_ITK_SHORT:
-            itk_image_save_short (img, (const char*) parms->output_fn);
-            break;
-        case PLM_IMG_TYPE_ITK_USHORT:
-            itk_image_save_ushort (img, (const char*) parms->output_fn);
-            break;
-        case PLM_IMG_TYPE_ITK_LONG:
-            itk_image_save_int32 (img, (const char*) parms->output_fn);
-            break;
-        case PLM_IMG_TYPE_ITK_ULONG:
-            itk_image_save_uint32 (img, (const char*) parms->output_fn);
-            break;
-        case PLM_IMG_TYPE_ITK_FLOAT:
-            itk_image_save_float (img, (const char*) parms->output_fn);
-            break;
-        case PLM_IMG_TYPE_ITK_DOUBLE:
-            itk_image_save_double (img, (const char*) parms->output_fn);
-            break;
-        default:
-            print_and_exit ("Output type is not supported.\n");
-            break;
-        }
+    if (parms->output_fn != "") {
+        itk_image_save (img, parms->output_fn, sm_parms->output_type);
     }
 
     /* ss_img */
-    if (parms->output_ss_img_fn.not_empty()) {
+    if (parms->output_ss_img_fn != "") {
 #if (PLM_CONFIG_USE_SS_IMAGE_VEC)
         rtds.get_rtss()->convert_to_uchar_vec ();
 #endif
@@ -91,19 +71,20 @@ do_synthetic_mha (Synthetic_mha_main_parms *parms)
     }
 
     /* dose_img */
-    if (parms->output_dose_img_fn.not_empty()) {
+    if (parms->output_dose_img_fn != "") {
         rtds.save_dose (parms->output_dose_img_fn);
     }
 
     /* list of structure names */
-    if (parms->output_ss_list_fn.not_empty()) {
+    if (parms->output_ss_list_fn != "") {
         printf ("save_ss_img: save_ss_list\n");
         rtds.get_rtss()->save_ss_list (parms->output_ss_list_fn);
     }
 
-    if (parms->output_dicom.not_empty()) {
+    if (parms->output_dicom != "") {
         rtds.get_rtss()->convert_ss_img_to_cxt ();
-        rtds.save_dicom ((const char*) parms->output_dicom);
+        rtds.save_dicom (parms->output_dicom.c_str(),
+            parms->dicom_with_uids);
     }
 
 }
@@ -148,12 +129,16 @@ parse_fn (
         "data type for output image: {uchar, short, ushort, ulong, float},"
         " default is float", 
         1, "float");
+    parser->add_long_option ("", "dicom-with-uids", 
+        "set to false to remove uids from created dicom filenames, "
+        "default is true", 1, "true");
 
     /* Main pattern */
     parser->add_long_option ("", "pattern",
         "synthetic pattern to create: {"
-        "donut, dose, gauss, grid, lung, noise, rect, sphere, "
-        "xramp, yramp, zramp}, default is gauss", 
+        "cylinder, donut, dose, gabor, gauss, grid, lung, noise, "
+        "rect, sphere, xramp, yramp, zramp"
+        "}, default is gauss", 
         1, "gauss");
 
     /* Image size */
@@ -184,6 +169,12 @@ parse_fn (
     parser->add_long_option ("", "donut-rings", 
         "number of donut rings (2 rings for traditional donut)", 1, "2");
         
+    /* Gabor options */
+    parser->add_long_option ("", "gabor-k-fib", 
+        "choose gabor direction at index i within fibonacci spiral "
+        "of length n; specified as \"i n\" where i and n are integers, "
+        "and i is between 0 and n-1", 1, "");
+
     /* Gaussian options */
     parser->add_long_option ("", "gauss-center", 
         "location of Gaussian center in mm \"x [y z]\"", 1, "0 0 0");
@@ -226,6 +217,12 @@ parse_fn (
     parser->add_long_option ("", "noise-std", 
         "standard deviation of gaussian noise", 1, "1.0");
 
+    /* Cylinder options */
+    parser->add_long_option ("", "cylinder-center", 
+        "location of cylinder center in mm \"x [y z]\"", 1, "0 0 0");
+    parser->add_long_option ("", "cylinder-radius", 
+        "size of cylinder in mm \"x [y z]\"", 1, "50 50 0");	
+
     /* Metadata options */
     parser->add_long_option ("", "metadata",
         "patient metadata (you may use this option multiple times)", 1, "");
@@ -253,16 +250,20 @@ parse_fn (
     Synthetic_mha_parms *sm_parms = &parms->sm_parms;
 
     /* Basic options */
-    parms->output_fn = parser->get_string("output").c_str();
-    parms->output_dicom = parser->get_string("output-dicom").c_str();
-    parms->output_dose_img_fn = parser->get_string("output-dose-img").c_str();
-    parms->output_ss_img_fn = parser->get_string("output-ss-img").c_str();
-    parms->output_ss_list_fn = parser->get_string("output-ss-list").c_str();
+    parms->output_fn = parser->get_string("output");
+    parms->output_dicom = parser->get_string("output-dicom");
+    parms->output_dose_img_fn = parser->get_string("output-dose-img");
+    parms->output_ss_img_fn = parser->get_string("output-ss-img");
+    parms->output_ss_list_fn = parser->get_string("output-ss-list");
     sm_parms->output_type = plm_image_type_parse (
-        parser->get_string("output-type").c_str());
+        parser->get_string("output-type"));
     if (sm_parms->output_type == PLM_IMG_TYPE_UNDEFINED) {
         throw dlib::error ("Error, unknown output-type\n");
     }
+    if (parser->option("dicom-with-uids")) {
+        parms->dicom_with_uids = string_value_true (
+            parser->get_string ("dicom-with-uids"));
+    }
 
     /* Main pattern */
     std::string arg = parser->get_string ("pattern");
@@ -302,12 +303,18 @@ parse_fn (
     else if (arg == "noise") {
         sm_parms->pattern = PATTERN_NOISE;
     }
+    else if (arg == "cylinder") {
+        sm_parms->pattern = PATTERN_CYLINDER;
+    }
+    else if (arg == "gabor") {
+        sm_parms->pattern = PATTERN_GABOR;
+    }
     else {
         throw (dlib::error ("Error. Unknown --pattern argument: " + arg));
     }
 
     /* Input files */
-    sm_parms->fixed_fn = parser->get_string("fixed").c_str();
+    sm_parms->fixed_fn = parser->get_string("fixed");
     sm_parms->input_fn = parser->get_string("input");
 
     /* Image size */
@@ -467,6 +474,16 @@ parse_fn (
                 "a floating point argument\n"));
     }
 
+    /* Cylinder options */
+    parser->assign_float_13 (sm_parms->cylinder_center, "cylinder-center");
+    parser->assign_float_13 (sm_parms->cylinder_radius, "cylinder-radius");
+
+    /* Gabor options */
+    if (parser->option ("gabor-k-fib")) {
+        sm_parms->gabor_use_k_fib = true;
+        parser->assign_int_2 (sm_parms->gabor_k_fib, "gabor-k-fib");
+    }
+
     /* Metadata options */
     for (unsigned int i = 0; i < parser->option("metadata").count(); i++) {
         parms->m_metadata.push_back (
