@@ -8,13 +8,18 @@
 #include <string.h>
 #include "itkCenteredTransformInitializer.h"
 #include "itkImageMaskSpatialObject.h"
+#include <itkImageMomentsCalculator.h>
 #include "itkLinearInterpolateImageFunction.h"
 #include "itkMutualInformationImageToImageMetric.h"
 #include "itkNormalizedMutualInformationHistogramImageToImageMetric.h"
 #include "itkRegularStepGradientDescentOptimizer.h"
-#if defined(ITK_USE_OPTIMIZED_REGISTRATION_METHODS)
+#if defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS)    \
+    && defined (PLM_CONFIG_USE_PATCHED_ITK)
 #include "plm_OptMattesMutualInformationImageToImageMetric.h"
 #include "itkOptMeanSquaresImageToImageMetric.h"
+#elif defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS)
+#include "itkOptMattesMutualInformationImageToImageMetric.h"
+#include "itkOptMeanSquaresImageToImageMetric.h"
 #else
 #include "itkMattesMutualInformationImageToImageMetric.h"
 #include "itkMeanSquaresImageToImageMetric.h"
@@ -42,9 +47,8 @@ typedef itk::MutualInformationImageToImageMetric <
     FloatImageType, FloatImageType > MIMetricType;
 typedef itk::NormalizedMutualInformationHistogramImageToImageMetric <
     FloatImageType, FloatImageType > NMIMetricType;
-/* modified Mattes mutual information class only available when 
-   using ITK_USE_OPTIMIZED_REGISTRATION_METHODS */
-#if defined(ITK_USE_OPTIMIZED_REGISTRATION_METHODS)
+#if defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS)    \
+    && defined (PLM_CONFIG_USE_PATCHED_ITK)
 typedef itk::plm_MattesMutualInformationImageToImageMetric <
     FloatImageType, FloatImageType > MattesMIMetricType;
 #else
@@ -61,7 +65,12 @@ typedef itk::LinearInterpolateImageFunction <
 static void
 itk_align_center (
     Registration_data* regd, Xform *xf_out, 
-    const Xform *xf_in, Stage_parms* stage);
+    const Xform *xf_in, const Stage_parms* stage);
+
+static void
+itk_align_center_of_gravity (
+    Registration_data* regd, Xform *xf_out, 
+    const Xform *xf_in, const Stage_parms* stage);
 
 Itk_registration_private::Itk_registration_private (
     Registration_data* regd, 
@@ -83,19 +92,26 @@ Itk_registration_private::~Itk_registration_private ()
     delete this->xf_best;
 }
 
-/* ITK throws exceptions when e.g. evaluating metrics with overlap 
-   of less that 25%.  Identify these so we can continue processing. */
 static bool
-itk_sample_failure (const itk::ExceptionObject& err)
+itk_unnecessary_exception (const itk::ExceptionObject& err)
 {
     std::string err_string = err.GetDescription();
-    const char *t = "Too many samples map outside moving image buffer";
 
-    if (err_string.find (t) != std::string::npos) {
+    /* ITK throws exceptions when evaluating metrics with overlap 
+       of less that 25%.  Identify these so we can continue processing. */
+    const char *t1 = "Too many samples map outside moving image buffer";
+    if (err_string.find (t1) != std::string::npos) {
         return true;
-    } else {
-        return false;
     }
+
+    /* ITK throws exceptions when MI joint PDF is zero.
+       Identify these so we can continue processing. */
+    const char *t2 = "Joint PDF summed to zero";
+    if (err_string.find (t2) != std::string::npos) {
+        return true;
+    }
+
+    return false;
 }
 
 double
@@ -108,7 +124,7 @@ Itk_registration_private::evaluate_initial_transform ()
             registration->GetInitialTransformParameters());
     }
     catch (itk::ExceptionObject & err) {
-        if (itk_sample_failure (err)) {
+        if (itk_unnecessary_exception (err)) {
             lprintf ("ITK failed with too few samples.\n");
             return value;
         }
@@ -182,14 +198,14 @@ Itk_registration_private::set_best_xform ()
 void
 Itk_registration_private::set_metric (FloatImageType::Pointer& fixed_ss)
 {
-    switch (stage->metric_type) {
-    case METRIC_MSE:
+    switch (stage->metric_type[0]) {
+    case REGISTRATION_METRIC_MSE:
     {
         MSEMetricType::Pointer metric = MSEMetricType::New();
         registration->SetMetric(metric);
     }
     break;
-    case METRIC_MI:
+    case REGISTRATION_METRIC_MI_VW:
     {
         /*  The metric requires a number of parameters to be
             selected, including the standard deviation of the
@@ -209,7 +225,7 @@ Itk_registration_private::set_metric (FloatImageType::Pointer& fixed_ss)
         registration->SetMetric(metric);
     }
     break;
-    case METRIC_MI_MATTES:
+    case REGISTRATION_METRIC_MI_MATTES:
     {
         /*  The metric requires two parameters to be selected: the 
             number of bins used to compute the entropy and the
@@ -225,11 +241,12 @@ Itk_registration_private::set_metric (FloatImageType::Pointer& fixed_ss)
             much higher proportion, such as $20$ percent. */
         MattesMIMetricType::Pointer metric = MattesMIMetricType::New();
         metric->SetNumberOfHistogramBins(
-            stage->mi_histogram_bins_fixed);
+            stage->mi_hist_fixed_bins);
         metric->SetNumberOfSpatialSamples (
             this->compute_num_samples (fixed_ss));
 
-#if defined(ITK_USE_OPTIMIZED_REGISTRATION_METHODS)
+#if defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS)    \
+    && defined (PLM_CONFIG_USE_PATCHED_ITK)
         /* Setting maxVal and minVal for MI calculation 
            (default==0 --> minVal and maxVal will be calculated from 
            images) */
@@ -242,7 +259,7 @@ Itk_registration_private::set_metric (FloatImageType::Pointer& fixed_ss)
         registration->SetMetric(metric);
     }
     break;
-    case METRIC_NMI:
+    case REGISTRATION_METRIC_NMI:
     {
         NMIMetricType::Pointer metric = NMIMetricType::New();
 
@@ -251,13 +268,15 @@ Itk_registration_private::set_metric (FloatImageType::Pointer& fixed_ss)
 #if defined (ITK_USE_REVIEW_STATISTICS) || (ITK_VERSION_MAJOR >= 4)
         hist.SetSize(2);
 #endif
-        hist[0] = stage->mi_histogram_bins_fixed;
-        hist[1] = stage->mi_histogram_bins_moving;
+        hist[0] = stage->mi_hist_fixed_bins;
+        hist[1] = stage->mi_hist_moving_bins;
         metric->SetHistogramSize (hist);
 
         /* Apparently sampling is not implemented in ITK 3 
            unless optimized registration methods are specified. */
-#if defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS) || (ITK_VERSION_MAJOR >= 4)
+#if ITK_VERSION_MAJOR >= 4                                        \
+    || (defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS)          \
+        && defined (PLM_CONFIG_USE_PATCHED_ITK))
         metric->SetNumberOfSpatialSamples (
             this->compute_num_samples (fixed_ss));
 #endif
@@ -591,13 +610,19 @@ Itk_registration_private::set_transform ()
     case STAGE_TRANSFORM_ALIGN_CENTER:
         set_transform_versor(registration, xf_out, xf_in, stage);
         break;
+    case STAGE_TRANSFORM_ALIGN_CENTER_OF_GRAVITY:
+        set_transform_versor(registration, xf_out, xf_in, stage);
+        break;
+    default:
+        print_and_exit ("Error: unknown case in set_transform()\n");
+        break;
     }
     registration->SetInitialTransformParameters (
         registration->GetTransform()->GetParameters());
 
     if (stage->xform_type != STAGE_TRANSFORM_BSPLINE) {
         std::stringstream ss;
-        ss << "Intial Parameters = " 
+        ss << "Initial Parameters = " 
             << registration->GetTransform()->GetParameters() << "\n";
         lprintf (ss.str().c_str());
     }
@@ -626,21 +651,26 @@ itk_registration_stage (
         return itk_align_center (regd, xf_out, xf_in, stage);
     }
 
+    /* align_center_of_gravity is handled separately */
+    if (stage->xform_type == STAGE_TRANSFORM_ALIGN_CENTER_OF_GRAVITY) {
+        return itk_align_center_of_gravity (regd, xf_out, xf_in, stage);
+    }
+
     Itk_registration_private irp (regd, xf_out, xf_in, stage);
     irp.registration = RegistrationType::New();
 
     /* Subsample fixed & moving images */
     FloatImageType::Pointer fixed_ss = subsample_image (
         regd->fixed_image->itk_float(), 
-        stage->fixed_subsample_rate[0], 
-        stage->fixed_subsample_rate[1], 
-        stage->fixed_subsample_rate[2], 
+        stage->resample_rate_fixed[0], 
+        stage->resample_rate_fixed[1], 
+        stage->resample_rate_fixed[2], 
         stage->default_value);
     FloatImageType::Pointer moving_ss = subsample_image (
         regd->moving_image->itk_float(), 
-        stage->moving_subsample_rate[0], 
-        stage->moving_subsample_rate[1], 
-        stage->moving_subsample_rate[2], 
+        stage->resample_rate_moving[0], 
+        stage->resample_rate_moving[1], 
+        stage->resample_rate_moving[2], 
         stage->default_value);
 
     irp.registration->SetFixedImage (fixed_ss);
@@ -665,7 +695,7 @@ itk_registration_stage (
         }
     }
     catch (itk::ExceptionObject & err) {
-        if (itk_sample_failure (err)) {
+        if (itk_unnecessary_exception (err)) {
             lprintf ("ITK failed with too few samples.\n");
         } else {
             lprintf ("Exception caught in itk registration.\n");
@@ -690,7 +720,7 @@ itk_registration_stage (
 static void
 itk_align_center (
     Registration_data* regd, Xform *xf_out, 
-    const Xform *xf_in, Stage_parms* stage)
+    const Xform *xf_in, const Stage_parms* stage)
 {
     float fixed_center[3];
     float moving_center[3];
@@ -704,6 +734,41 @@ itk_align_center (
     xf_out->set_trn (trn_parms);
 }
 
+static void
+itk_align_center_of_gravity (
+    Registration_data* regd, Xform *xf_out, 
+    const Xform *xf_in, const Stage_parms* stage)
+{
+
+    if (regd->fixed_roi != NULL && regd->moving_roi != NULL) {
+        typedef itk::ImageMomentsCalculator<UCharImageType> ImageMomentsCalculatorType;
+        
+        ImageMomentsCalculatorType::Pointer fixedCalculator = ImageMomentsCalculatorType::New();
+        fixedCalculator->SetImage(regd->fixed_roi->itk_uchar());
+        fixedCalculator->Compute();
+
+        ImageMomentsCalculatorType::Pointer movingCalculator = ImageMomentsCalculatorType::New();
+        movingCalculator->SetImage(regd->moving_roi->itk_uchar());
+        movingCalculator->Compute();
+
+        ImageMomentsCalculatorType::VectorType fixedCenter; 
+        ImageMomentsCalculatorType::VectorType movingCenter; 
+    
+        fixedCenter = fixedCalculator->GetCenterOfGravity();
+        movingCenter = movingCalculator->GetCenterOfGravity();
+
+        itk::Array<double> trn_parms (3);
+        trn_parms[0] = movingCenter[0] - fixedCenter[0];
+        trn_parms[1] = movingCenter[1] - fixedCenter[1];
+        trn_parms[2] = movingCenter[2] - fixedCenter[2];
+        xf_out->set_trn (trn_parms);
+    }
+
+    else {
+        print_and_exit("NO ROIs SET!");
+    }
+}
+
 /* Greg's notes about itk problems (ITK 3.20.1)
    There are two ways to set the ITK bspline parameters:
 
