@@ -22,10 +22,10 @@
 #include "path_util.h"
 #include "plm_image.h"
 #include "plm_image_header.h"
-#include "plm_stages.h"
 #include "plm_warp.h"
 #include "print_and_exit.h"
 #include "string_util.h"
+#include "registration.h"
 #include "registration_parms.h"
 #include "registration_data.h"
 #include "rt_study.h"
@@ -52,7 +52,7 @@ Mabs_atlas_selection::Mabs_atlas_selection ()
     this->atlases_from_ranking = -1;
     this->number_of_atlases = -1;
     this->hist_bins = 100;
-    this->mask = MaskType::New();
+    this->percentage_nmi_random_sample = -1;
     this->min_hist_sub_value_defined = false;
     this->min_hist_sub_value=0;
     this->max_hist_sub_value_defined = false;
@@ -238,26 +238,19 @@ Mabs_atlas_selection::compute_general_similarity_value()
 double
 Mabs_atlas_selection::compute_similarity_value_post()
 {
-    
-    Registration_parms* regp = new Registration_parms;
-    Registration_data* regd = new Registration_data;
+    Registration reg;
+    Registration_parms::Pointer regp = reg.get_registration_parms ();
+    Registration_data::Pointer regd = reg.get_registration_data ();
 
-    regp->parse_command_file(this->selection_reg_parms_fn.c_str());
-    
-    regd->fixed_image = this->subject;
-    regd->moving_image = this->atlas;
-
-    /* Make sure to have just the right inputs */
-    regd->fixed_landmarks = NULL;
-    regd->moving_landmarks = NULL;
+    reg.set_command_file (this->selection_reg_parms_fn);
+    reg.set_fixed_image (this->subject);
+    reg.set_moving_image (this->atlas);
+    Xform::Pointer xf = reg.do_registration_pure ();
 
     Plm_image::Pointer deformed_atlas = Plm_image::New ();
     Plm_image_header fixed_pih (regd->fixed_image);
-
-    Xform::Pointer xf = do_registration_pure (regd, regp);
     plm_warp (deformed_atlas, 0, xf, &fixed_pih, 
-        regd->moving_image,
-        regp->default_value, 0, 1);
+        this->atlas, regp->default_value, 0, 1);
    
     double similarity_value_post = 0;
     if (this->atlas_selection_criteria == "nmi-post") {
@@ -270,9 +263,6 @@ Mabs_atlas_selection::compute_similarity_value_post()
         lprintf("MSE post = %g \n", similarity_value_post);
     }
    
-    delete regd;
-    delete regp;
-
     return similarity_value_post;
 }
 
@@ -288,27 +278,21 @@ Mabs_atlas_selection::compute_similarity_value_ratio()
         similarity_value_pre = this->compute_mse(this->subject, this->atlas);
    
     lprintf("Similarity value pre = %g \n", similarity_value_pre);
-    
-    Registration_parms* regp = new Registration_parms;
-    Registration_data* regd = new Registration_data;
 
-    regp->parse_command_file(this->selection_reg_parms_fn.c_str());
-    
-    regd->fixed_image = this->subject;
-    regd->moving_image = this->atlas;
+    Registration reg;
+    Registration_parms::Pointer regp = reg.get_registration_parms ();
+    Registration_data::Pointer regd = reg.get_registration_data ();
 
-    /* Make sure to have just the right inputs */
-    regd->fixed_landmarks = NULL;
-    regd->moving_landmarks = NULL;
+    reg.set_command_file (this->selection_reg_parms_fn);
+    reg.set_fixed_image (this->subject);
+    reg.set_moving_image (this->atlas);
+    Xform::Pointer xf = reg.do_registration_pure ();
 
     Plm_image::Pointer deformed_atlas = Plm_image::New ();
     Plm_image_header fixed_pih (regd->fixed_image);
-
-    Xform::Pointer xf = do_registration_pure (regd, regp);
     plm_warp (deformed_atlas, 0, xf, &fixed_pih, 
-        regd->moving_image,
-        regp->default_value, 0, 1);
-    
+        this->atlas, regp->default_value, 0, 1);
+
     double similarity_value_post = 0;
 
     if (this->atlas_selection_criteria == "nmi-ratio")
@@ -317,9 +301,6 @@ Mabs_atlas_selection::compute_similarity_value_ratio()
         similarity_value_post = this->compute_mse (this->subject, deformed_atlas);
    
     lprintf("Similarity value post = %g \n", similarity_value_post);
-    
-    delete regd;
-    delete regp;
 
     return ((similarity_value_post/similarity_value_pre)-1) * similarity_value_post;
 }
@@ -350,7 +331,26 @@ Mabs_atlas_selection::compute_nmi (
     /* Set mask if defined */
     if (this->mask)
     {
-       	nmi_metric->SetFixedImageMask(this->mask);	
+       	nmi_metric->SetFixedImageMask(this->mask);
+    }
+
+    /* Set number of random sample if defined */
+    if (this->percentage_nmi_random_sample != -1) { /* User set this value */
+        if (this->percentage_nmi_random_sample <= 0 ||
+            this->percentage_nmi_random_sample > 1) { /* The set value is not in the range */
+            printf("Percentage nmi random sample not set properly. " 
+                   "User setting will be ignored and the default value will be used\n");
+        }
+
+        else { /* The set value is in the range, apply it */
+            unsigned long number_of_img_voxels =
+                img1->itk_short()->GetLargestPossibleRegion().GetNumberOfPixels();
+            unsigned long number_spatial_samples =
+                static_cast<unsigned long> (number_of_img_voxels * this->percentage_nmi_random_sample);
+#if defined (ITK_USE_OPTIMIZED_REGISTRATION_METHODS) || (ITK_VERSION_MAJOR >= 4)
+            nmi_metric->SetNumberOfSpatialSamples(number_spatial_samples);
+#endif
+        }
     }
     
     /* Set histogram interval if defined */
@@ -395,7 +395,7 @@ Mabs_atlas_selection::compute_nmi (
     nmi_metric->SetInterpolator(interpolator);
     
     nmi_metric->Initialize();
-    
+
     /* NMI compute */
     return (double) nmi_metric->GetValue(transform->GetParameters());
 }
