@@ -12,6 +12,7 @@
 #include "itk_directions.h"
 #include "itk_image_type.h"
 #include "itk_point.h"
+#include "logfile.h"
 #include "plm_image.h"
 #include "plm_image_header.h"
 #include "plm_math.h"
@@ -20,10 +21,11 @@
 #include "segmentation.h"
 #include "synthetic_mha.h"
 
-
 class Synthetic_mha_parms_private {
 public:
     vnl_random v;
+    float gabor_freq;
+    float gabor_proj[3];
 };
 
 
@@ -46,6 +48,8 @@ Synthetic_mha_parms::Synthetic_mha_parms ()
         donut_center[i] = 0.0f;
         lung_tumor_pos[i] = 0.0f;
         dose_center[i] = 0.0f;
+        cylinder_center[i] = 0.0f;
+        cylinder_radius[i] = 0.0f;
     }
     background = -1000.0f;
     foreground = 0.0f;
@@ -53,6 +57,7 @@ Synthetic_mha_parms::Synthetic_mha_parms ()
     foreground_alpha = 1.0f;
     m_want_ss_img = false;
     m_want_dose_img = false;
+    image_normalization = NORMALIZATION_NONE;
     rect_size[0] = -50.0f;
     rect_size[1] = +50.0f;
     rect_size[2] = -50.0f;
@@ -76,6 +81,12 @@ Synthetic_mha_parms::Synthetic_mha_parms ()
     num_multi_sphere = 33;
     noise_mean = 0;
     noise_std = 1.f;
+    gabor_use_k_fib = false;
+    gabor_k_fib[0] = 0;
+    gabor_k_fib[1] = 1;
+    gabor_k[0] = 1.;
+    gabor_k[1] = 0.;
+    gabor_k[2] = 0.;
 }
 
 Synthetic_mha_parms::~Synthetic_mha_parms ()
@@ -221,10 +232,10 @@ synth_gauss (
     const Synthetic_mha_parms *parms
 )
 {
-    float f = 0;
+    double f = 0;
     for (int d = 0; d < 3; d++) {
-        float f1 = phys[d] - parms->gauss_center[d];
-        f1 = f1 / parms->gauss_std[d];
+        double f1 = phys[d] - (double) parms->gauss_center[d];
+        f1 = f1 / (double) parms->gauss_std[d];
         f += f1 * f1;
     }
     f = exp (-0.5 * f);            /* f \in (0,1] */
@@ -476,9 +487,69 @@ synth_noise (
     *intens = parms->noise_mean + (float) r;
 }
 
+
+static void 
+synth_cylinder (
+    float *intens, 
+    unsigned char *label,
+    const FloatPoint3DType& phys, 
+    const Synthetic_mha_parms *parms
+)
+{
+    float f = 0;
+    for (int d = 0; d < 2; d++) {
+        float f1 = phys[d] - parms->cylinder_center[d];
+        f1 = f1 / parms->cylinder_radius[d];
+        f += f1 * f1;
+    }
+    if (f > 1.0) {
+        *intens 
+            = (1 - parms->background_alpha) * (*intens) 
+            + parms->background_alpha * parms->background;
+        *label = 0;
+    } else {
+        *intens 
+            = (1 - parms->foreground_alpha) * (*intens) 
+            + parms->foreground_alpha * parms->foreground;
+        *label = 1;
+    }	
+}
+
+static void 
+synth_gabor (
+    float *intens, 
+    unsigned char *label,
+    const FloatPoint3DType& phys, 
+    const Synthetic_mha_parms *parms
+)
+{
+    /* Do gaussian first */
+    float f = 0;
+    float rel[3];
+    for (int d = 0; d < 3; d++) {
+        rel[d] = phys[d] - parms->gauss_center[d];
+        float f1 = rel[d] / parms->gauss_std[d];
+        f += f1 * f1;
+    }
+    f = exp (-0.5 * f);            /* f \in (0,1] */
+
+    *intens = (1 - f) * parms->background + f * parms->foreground;
+    *label = (f > 0.2) ? 1 : 0;
+
+    /* Get the distance from the central plane in direction of 
+       projection vector ("k" vector) */
+    float proj_dist = 0.f;
+    for (int d = 0; d < 3; d++) {
+        proj_dist += rel[d] * parms->d_ptr->gabor_proj[d];
+    }
+
+    /* Modulate by cos for real part, or sin for imaginary part */
+    *intens *= cos(parms->d_ptr->gabor_freq * proj_dist);
+}
+
 void
 synthetic_mha (
-    Rt_study *rtds,
+    Rt_study *rt_study,
     Synthetic_mha_parms *parms
 )
 {
@@ -503,7 +574,7 @@ synthetic_mha (
 
     } else {
         /* No input image specified */
-        if (parms->fixed_fn.not_empty()) {
+        if (parms->fixed_fn != "") {
             /* Get geometry from fixed image */
             Plm_image pi (parms->fixed_fn);
             Plm_image_header pih;
@@ -570,6 +641,54 @@ synthetic_mha (
         dose_img_it.GoToBegin();
     }
 
+    /* Figure out Gabor settings */
+    if (parms->pattern == PATTERN_GABOR) {
+        /* Figure out projection direction */
+        if (parms->gabor_use_k_fib) {
+            /* Fibonacci method selected. */
+            /* N.b. I'm not sure which is the better value for phi.
+               So let's use the golden section value. */
+            //float phi = (3. - sqrt(5.)) * M_PI;
+            float phi = (((sqrt(5.) + 1.) / 2.) - 1.) * M_PI;
+            float longitude = fmodf (phi * parms->gabor_k_fib[0], 2 * M_PI);
+            float latitude_sin = -1. + 2. * ((float) parms->gabor_k_fib[0]) / 
+                parms->gabor_k_fib[1];
+            float latitude = asinf (latitude_sin);
+            float latitude_cos = cos (latitude);
+            parms->d_ptr->gabor_proj[0] = latitude_cos * cos (longitude);
+            parms->d_ptr->gabor_proj[1] = latitude_cos * sin (longitude);
+            parms->d_ptr->gabor_proj[2] = latitude_sin;
+            
+            lprintf ("Gabor: %d %d -> (%f %f) -> (%f %f %f)\n", 
+                parms->gabor_k_fib[0],
+                parms->gabor_k_fib[1],
+                longitude, latitude, 
+                parms->d_ptr->gabor_proj[0],
+                parms->d_ptr->gabor_proj[1],
+                parms->d_ptr->gabor_proj[2]);
+        }
+        else {
+            /* Standard "k" vector supplied.  Need to normalize it. */
+            float len = vec3_len (parms->gabor_k);
+            if (len < FLOAT_SMALL_VECTOR_LENGTH) {
+                parms->gabor_k[0] = 1.;
+                parms->gabor_k[1] = 0.;
+                parms->gabor_k[2] = 0.;
+                len = 1.;
+            }
+            vec3_scale3 (parms->d_ptr->gabor_proj, parms->gabor_k, len);
+        }
+
+        /* Automatically set frequency based on width.  
+           The value 1.5 is a heuristic. */
+        float sum = 0.f;
+        for (int d = 0; d < 3; d++) {
+            sum += parms->gauss_std[d] * parms->gauss_std[d];
+        }
+        parms->d_ptr->gabor_freq = M_PI * 1.5 / sqrt(sum);
+        parms->image_normalization = Synthetic_mha_parms::NORMALIZATION_GABOR;
+    }
+
     /* Iterate through image, setting values */
     typedef itk::ImageRegionIteratorWithIndex< FloatImageType > IteratorType;
     IteratorType it_out (im_out, im_out->GetLargestPossibleRegion());
@@ -617,6 +736,12 @@ synthetic_mha (
         case PATTERN_NOISE:
             synth_noise (&intens, &label_uchar, phys, parms);
             break;
+        case PATTERN_CYLINDER:
+            synth_cylinder (&intens, &label_uchar, phys, parms);
+            break;
+        case PATTERN_GABOR:
+            synth_gabor (&intens, &label_uchar, phys, parms);
+            break;
         default:
             intens = 0.0f;
             label_uchar = 0;
@@ -653,15 +778,97 @@ synthetic_mha (
         }
     }
 
-    /* Insert images into rtds */
+    /* Normalize if requested */
+    if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_SUM_ONE) {
+        float sum = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            sum += it_out.Get();
+        }
+        if (sum > 1e-10) {
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                it_out.Set(it_out.Get() / sum);
+            }
+        }
+    }
+    else if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_SUM_SQR_ONE) {
+        float sum = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get();
+            sum += v * v;
+        }
+        sum = sqrt(sum);
+        if (sum > 1e-10) {
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                it_out.Set(it_out.Get() / sum);
+            }
+        }
+    }
+    else if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_ZERO_MEAN_STD_ONE) {
+        /* GCS FIX: I should really compute in a single pass.
+           Oh God almighty, when did I get so lazy? */
+        int num_vox = 0;
+        float sum_1 = 0.f, sum_2 = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get();
+            sum_1 += v;
+            num_vox ++;
+        }
+        sum_1 = sum_1 / num_vox;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get() - sum_1;
+            it_out.Set(v);
+            sum_2 += v * v;
+        }
+        sum_2 = sqrt(sum_2);
+        if (sum_2 > 1e-10) {
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                it_out.Set(it_out.Get() / sum_2);
+            }
+        }
+    }
+    else if (parms->image_normalization == Synthetic_mha_parms::NORMALIZATION_GABOR) {
+        float sum_lo_1 = 0.f, sum_hi_1 = 0.f;
+        float sum_lo_2 = 0.f, sum_hi_2 = 0.f;
+        for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+            float v = it_out.Get();
+            if (v < 0) {
+                sum_lo_1 += v;
+                sum_lo_2 += v * v;
+            } else if (v > 0) {
+                sum_hi_1 += v;
+                sum_hi_2 += v * v;
+            }
+        }
+        sum_lo_2 = M_SQRT1_2 * sqrt(sum_lo_2);
+        sum_hi_2 = M_SQRT1_2 * sqrt(sum_hi_2);
+        float sum_lo_3 = 0.f, sum_hi_3 = 0.f;
+        if (sum_lo_2 > 1e-10 && sum_hi_2 > 1e-10) {
+            if (sum_lo_2 < sum_hi_2) {
+                sum_lo_2 = sum_hi_2 * -sum_lo_1 / sum_hi_1;
+            } else {
+                sum_hi_2 = sum_lo_2 * -sum_hi_1 / sum_lo_1;
+            }
+            for (it_out.GoToBegin(); !it_out.IsAtEnd(); ++it_out) {
+                float v = it_out.Get();
+                if (v < 0) {
+                    it_out.Set(v / sum_lo_2);
+                    sum_lo_3 += v / sum_lo_2;
+                } else if (v > 0) {
+                    it_out.Set(v / sum_hi_2);
+                    sum_hi_3 += v / sum_hi_2;
+                }
+            }
+        }
+    }
+
+    /* Insert images into rt_study */
     Plm_image::Pointer pli = Plm_image::New();
     pli->set_itk (im_out);
-    rtds->set_image (pli);
+    rt_study->set_image (pli);
     if (parms->m_want_ss_img) {
-        /* Create rtss & set into rtds */
-        Segmentation::Pointer rtss = Segmentation::New (
-            new Segmentation (rtds));
-        rtds->set_rtss (rtss);
+        /* Create rtss & set into rt_study */
+        Segmentation::Pointer rtss = Segmentation::New ();
+        rt_study->set_rtss (rtss);
 
         /* Insert ss_image into rtss */
         rtss->set_ss_img (ss_img);
@@ -673,19 +880,19 @@ synthetic_mha (
         /* Add structure names */
         switch (parms->pattern) {
         case PATTERN_LUNG:
-            rtss_ss->add_structure (Pstring ("Body"),
-                Pstring(), 1, 0);
-            rtss_ss->add_structure (Pstring ("Tumor"),
-                Pstring(), 2, 1);
-            rtss_ss->add_structure (Pstring ("Lung"),
-                Pstring(), 3, 2);
+            rtss_ss->add_structure (
+                std::string ("Body"), std::string(), 1, 0);
+            rtss_ss->add_structure (
+                std::string ("Tumor"), std::string(), 2, 1);
+            rtss_ss->add_structure (
+                std::string ("Lung"), std::string(), 3, 2);
             break;
         default:
-            rtss_ss->add_structure (Pstring ("Foreground"),
-                Pstring(), 1, 0);
+            rtss_ss->add_structure (
+                std::string ("Foreground"), std::string(), 1, 0);
         }
     }
     if (parms->m_want_dose_img) {
-        rtds->set_dose (dose_img);
+        rt_study->set_dose (dose_img);
     }
 }
